/*
 * arch/arm/mach-iop33x/include/mach/iq80332.h
 *
 * Intel IQ80332 evaluation board registers
 */

#ifndef __IQ80332_H
#define __IQ80332_H

#define IQ80332_7SEG_1		0xce840000	/* 7-Segment MSB */
#define IQ80332_7SEG_0		0xce850000	/* 7-Segment LSB (WO) */
#define IQ80332_ROTARY_SW	0xce8d0000	/* Rotary Switch */
#define IQ80332_BATT_STAT	0xce8f0000	/* Battery Status */


#endif
