/*
 * arch/arm/mach-prima2/include/mach/timex.h
 *
 * Copyright (c) 2011 Cambridge Silicon Radio Limited, a CSR plc group company.
 *
 * Licensed under GPLv2 or later.
 */

#ifndef __MACH_TIMEX_H__
#define __MACH_TIMEX_H__

#define CLOCK_TICK_RATE  1000000

#endif
